import datetime
import os
import sys

import pandas as pd

from paths import tabulated, tabulated_merge,look_up_use,overwrite_inter_data, pctn_table, in_pol_bndry_overlap, st_abb_lookup, grouping_col


# Title- Generate overlap tables from zone species rasters to use layers results;

# ASSUMPTIONS
# Species group is found in index position 1 of all input result tables when split by '_'
# All raster are 30-meter cells - note previously VI and CNMI has some use with a different cell size

# ###Functions


def create_directory(dbf_dir):
    if not os.path.exists(dbf_dir):
        os.mkdir(dbf_dir)


def state_pct(row, msq_state_col, pct_col):
    # calc state treated area
    adjust_value = row[msq_state_col] * row[pct_col]
    return adjust_value


def outside_range(row, state, species_state):
    # calc use area outside species files
    adjust_value = row[state] - row[species_state]
    # address edge effect when there is more area in the species range in a state than in the whole state updated on
    # 7/30/20 lower limited of outside the range but with in the state is 0; edge effect can result in negative
    # values
    if adjust_value < 0:
        adjust_value = 0
    return adjust_value

def uniform_spe(row, pct, sp_total_col):
    # calc uniform treated area in species files
    acres = row[sp_total_col]
    return acres * row[pct]


def min_range(row, total, adjust):
    # calc min use area in species files
    total_outside = row[total]
    st_adjust = row[adjust]
    if total_outside > st_adjust:
        return 0
    else:
        return st_adjust - total_outside


def max_range(row, total, adjust, ):
    # calc  max use area in species files
    st_adjust = row[adjust]
    sp_total = row[total]
    if sp_total < st_adjust:
        return sp_total
    else:
        return st_adjust


def classify_drift(row, col, sp_total):
    if row[sp_total] == 0:  # can't divide by 0, if the overlap is 0 drift ratio would also be 0
        return 0
    else:
        ratio = row[col]/row[sp_total]
    # if ratio shouldn't be more tha 1, when it is caused by spatial anomalies when summarizing to political boundaries
    if ratio <= 1:
        return ratio
    else:
        return 1

def apply_usage(in_folder ,csv, use_lookup, use, region_name, merged_df,t_suffix):
        df = pd.read_csv(in_folder +os.sep + csv )
        index_cols = [t for t in df.columns.values.tolist() if t.startswith('Unnamed')]
        if len(index_cols) > 0:
            df.drop(index_cols, axis=1, inplace=True)
        overlap_cols = [v for v in df.columns.tolist() if v.startswith("VALUE")]
        df[overlap_cols] = df[overlap_cols].apply(pd.to_numeric, errors='coerce')

        print ("Working on use {0} in region {1}".format(use, region_name))
        # print csv

        df_col = ['EntityID','STUSPS'] + overlap_cols
        df = df[df_col].copy()
        # sums county numbers to state if counties were include - needed to apply usage
        if csv.endswith('_Cnty.csv'):
            df = df[df_col].copy()
            state = df.groupby(['EntityID','STUSPS'], as_index=False).sum().reset_index().copy()
        else:
            state = df.copy()
        # overlaps with state

        merged_df = pd.merge(state,merged_df,how='left', on=['STUSPS'])

        # PCT Adjustments by treated area bounding scenario
        # calculates the state treated area
        for col in num_pct:
            pct = col.split("_")[1].upper()
            # calculated the total use outside the species range, min use in range (lower bound
            # distribution of treated areas); max in species range (upper bound distribution of
             # treated areas) and the uniform distribution of treated areas
            # calculates the uniform treated area by applying the PCT to species results
            merged_df['Uniform MSQ'] = merged_df.apply(lambda row: uniform_spe(row, col, 'VALUE_0'),axis=1)
            merged_df['State MSQ adjusted by PCT'] = merged_df.apply(lambda row: state_pct(row, 'State MSQ', col), axis=1)
            merged_df['Total outside species range'] = merged_df.apply(lambda row: outside_range(row, 'State MSQ', 'VALUE_0'), axis=1)
            merged_df['Upper in species range'] = merged_df.apply(lambda row: max_range(row,  'State MSQ adjusted by PCT','VALUE_0'), axis=1)
            merged_df ['Drift Ratio_Upper_'+ use] = merged_df.apply(lambda row: classify_drift(row, 'State MSQ adjusted by PCT', 'VALUE_0'), axis=1)
            merged_df['Drift Ratio_Uniform_' + use] = merged_df.apply(lambda row: classify_drift(row, 'Uniform MSQ', 'VALUE_0'), axis=1)

            drift_col = [v for v in merged_df.columns.values.tolist() if v.startswith("VALUE")]
            drift_col.remove("VALUE_0")
            #merged_df.to_csv(out_folder + os.sep + out_csv + pct + 'Upper_State_PCTn.csv', encoding='ISO-8859-1')

            out_uniform = merged_df.copy()
            out_uniform ['VALUE_0'] = out_uniform ['Uniform MSQ']
            out_uniform[drift_col]= out_uniform[drift_col].multiply(out_uniform['Drift Ratio_Uniform_'+use], axis=0)
            out_uniform = out_uniform[df_col + ['Drift Ratio_Uniform_'+use]].copy()
            out_uniform.fillna(0,inplace=True)  # blank values are set to 0 - blank means there is no overlap

            out_upper = merged_df.copy()
            out_upper['VALUE_0'] = out_upper['Upper in species range']
            out_upper[drift_col]= out_upper[drift_col].multiply(out_upper['Drift Ratio_Upper_' + use], axis=0)
            out_upper = out_upper[df_col + ['Drift Ratio_Upper_' + use]].copy()
            out_upper .fillna(0,inplace=True)  # blank values are set to 0 - blank means there is no overlap
            out_name = csv.replace(t_suffix,"")
            out_upper.to_csv(out_folder + os.sep + out_name + "_Upper_State_"+ pct+'_PCTn.csv', encoding='ISO-8859-1')
            out_uniform.to_csv(out_folder + os.sep + out_name + "_Uniform_State_"+ pct+'_PCTn.csv', encoding='ISO-8859-1')

            # merged_df['Lower in species range'] = merged_df.apply(lambda row: min_range(row, 'Total outside species range', 'Total outside species range'), axis=1)
            #out_lower = merged_df.copy()
            #out_lower ['VALUE_0'] = out_uniform ['Lower in species range']
            # out_lower.to_csv(out_folder + os.sep + out_csv + "_Lower" + pct + '_State_PCTn.csv', encoding='ISO-8859-1')



start_time = datetime.datetime.now()
print ("Start Time: " + start_time.ctime())

# sets the folder with where the summary files will be found


in_folder = tabulated_merge
if not os.path.dirname(in_folder):
    print('Cannot find the merged folder in the output directory; confirm input values on the paths.py')
    sys.exit()
else:
    # Set output
    # For future up change output table to indicate state
    # if os.path.basename(tabulated_merge) == "Overlap_byCountyMerge":
    #     out_folder = os.path.dirname(tabulated_merge) +os.sep+"Overlap_byStateMerge_PCTns"
    # else:
    #     out_folder = tabulated_merge + "_PCTns"
    out_folder = tabulated_merge + "_PCTns"
    if not os.path.exists(out_folder):
        os.mkdir(out_folder)

# date
today = datetime.datetime.today()
date = today.strftime('%Y%m%d')

state_cnty_total = os.listdir(in_pol_bndry_overlap)
state_cnty_total = [v for v in state_cnty_total if v.startswith('StateAcres')]
# load chemical use
use_lookup = pd.read_csv(look_up_use, low_memory=False, encoding = "ISO-8859-1")

if 'PCTn' in use_lookup.columns.values.tolist():
    pctn = use_lookup.loc[use_lookup['PCTn'] == 'x']
    pctn_uses = pctn['FinalUseHeader'].values.tolist()
    pctn_uses = sorted(list(set(pctn_uses)))
    if len(pctn_uses) < 1:
        print(
            "You have not identified PCTns for any of your uses in the PCTn column if the use look-up table. \nConfirm you have PCTns or PCT values to include in you overlap overlap. If you do not have usage to include in the overlap move to the next step. This is an optional step if you have usage.")
        sys.exit()
all_csv_files = []
for use in pctn_uses:
    print("You have a PCTn for {0}".format(use))
    name = use_lookup.loc[use_lookup['FinalUseHeader'] == use, 'FullName'].iloc[0]
    csv = [v for v in os.listdir(in_folder) if v.startswith(name)]
    all_csv_files.append(csv[0])


if "GEOID" in grouping_col:
    pol_type = '_Cnty.csv'
else:
    pol_type = '_State.csv'


for uses_list in pctn_uses:
    for csv in all_csv_files:
        out_csv = csv.replace(pol_type, '')
        use = use_lookup.loc[use_lookup['FullName'] == (csv.replace("_euc" + pol_type, "")), 'FinalUseHeader'].iloc[0]
        region_use = use_lookup.loc[use_lookup['FullName'] == (csv.replace("_euc" + pol_type, "")), 'Region'].iloc[0]

        if uses_list != use:
            continue
        # Extract State Totals to generate usage adjustments
        state_total = [v for v in state_cnty_total if v.endswith(use + '.csv')]
        state_total_df = pd.read_csv(in_pol_bndry_overlap + os.sep + state_total[0], thousands=',')
        state_total_df['Acres'] = state_total_df['Acres'].apply(pd.to_numeric, errors='coerce')
        state_total_df['State MSQ'] = state_total_df['Acres'] * 4046.86
        state_total_df['STATEFP'] = state_total_df['STATEFP'].map(lambda x: str(x).split('.')[0]).astype(int)
        # read PCTn table
        pctn_df = pd.read_csv(pctn_table)
        filter_pct = ['State', 'STUSPS']
        # print(filter_pct)
        num_pct = [v for v in pctn_df.columns.values.tolist() if v.startswith(use)]
        filter_pct.extend(num_pct)
        pctn_df_use = pctn_df[filter_pct].copy()

        if not "STUSPS" in state_total_df.columns.values.tolist() and 'STATEFP' in state_total_df.columns.values.tolist():
            st_abb_df = pd.read_csv(st_abb_lookup)
            st_abb_df = st_abb_df.reindex(columns=['STATEFP', 'STUSPS'])
            st_abb_df = st_abb_df[['STATEFP', 'STUSPS']].copy()
            st_abb_df.drop_duplicates(inplace=True)
            state_total_df = pd.merge(state_total_df, st_abb_df, how='left', on='STATEFP')

        if not "STUSPS" in state_total_df.columns.values.tolist():
            print("Check your state overlap table it is missing the state abbreviate")
            sys.exit()

        merged_df = pd.merge(pctn_df_use, state_total_df, how='left', on='STUSPS')
        merged_df.drop_duplicates(inplace=True)

        if use == "Ag":
            out_csv = out_csv.replace("AK", "NL48")
            merged_df.to_csv(out_folder + os.sep + out_csv + '_State_merged.csv', encoding='ISO-8859-1')
            nl48_regions = ['HI', 'AK', 'AS', 'CNMI', 'GU', 'PR', 'VI']
            for r in nl48_regions:
                region_use = r
                csv_region = csv.split("_")
                csv_updated = r + "_" + csv_region[1] + "_" + csv_region[2] + "_" + csv_region[3]
                apply_usage(in_folder, csv_updated, use_lookup, use, region_use, merged_df,pol_type)
        else:
            merged_df.to_csv(out_folder + os.sep + out_csv + '_State_merged.csv', encoding='ISO-8859-1')
            apply_usage(in_folder, csv, use_lookup, use, region_use, merged_df,pol_type )

end = datetime.datetime.now()
print("End Time: " + end.ctime())
elapsed = end - start_time
print("Elapsed  Time: " + str(elapsed))


