import datetime
import os
import sys

import pandas as pd

from paths import full_impact
from paths import regions, col_include_output, look_up_use, bins, master_list, final_path, range_run,coa_usage_excel, python_version_three

if python_version_three:
    from warnings import simplefilter
    simplefilter(action="ignore", category=pd.errors.PerformanceWarning)

# Title- Generate drift interval summary table from percent overlap tables

# ASSUMPTION: Cannot have "_" in the use name part of final use col headers; set in the use look-up table
# ie [region]_[use name]_[interval_value] list of current group of column will not populate correctly


# Elapse clock
start_time = datetime.datetime.now()
print ("Start Time: " + start_time.ctime())

final_path = final_path +os.sep+"ParentTables"

# CoA usage
# Loads inputs if errors occur confirm sheet names in the excel workbook and header position

include_coa=False
if os.path.exists(coa_usage_excel):
    include_coa=True
    try:
        coa_col = ['Common Name', 'Scientific Name', 'ESA Listing Status',	'Lead Agency','Species Group']
        if python_version_three:
            if range_run:
                coa_usage = pd.read_excel(coa_usage_excel, sheet_name="Species", header=0, engine='openpyxl')
            else:
                coa_usage = pd.read_excel(coa_usage_excel, sheet_name="Critical Habitat", header=0, engine='openpyxl')
        else:
            if range_run:
                coa_usage = pd.read_excel(coa_usage_excel, sheetname="Species", header=0, encoding="utf-8")
            else:
                coa_usage = pd.read_excel(coa_usage_excel, sheetname="Critical Habitat", header=0, encoding="utf-8")
        coa_usage.drop(coa_col, axis=1, inplace=True)
        coa_usage["EntityID"] = coa_usage["EntityID"].map(lambda x: str(x).split(".")[0]).astype(str)
           
            
    except Exception as error:
        # if it can't access CoA usage file able it will print this message and exit the script
        print ("Could not load the coa usage excel workbook, please confirm the file name on the paths.py")
        print(error.args[0])
        sys.exit(())
                                  
# date
today = datetime.datetime.today()
date = today.strftime('%Y%m%d')
# chemical use look-up able
use_lookup = pd.read_csv(look_up_use,encoding ='ISO-8859-1')
use_lookup['FinalColHeader'].fillna('none', inplace=True)
region_lookup = use_lookup.loc[use_lookup['Region'].isin(regions)]
species_df = pd.read_csv(master_list, dtype=object,low_memory=False,encoding ='ISO-8859-1')
[species_df.drop(m, axis=1, inplace=True) for m in species_df.columns.values.tolist() if m.startswith('Unnamed')]
species_df['EntityID'] = species_df['EntityID'].map(lambda k: str(k).split('.')[0]).astype(str)
base_sp_df = species_df[col_include_output].copy()
if len(list(set(region_lookup['Region'].values.tolist())))>1:
    region_sum = ['CONUS', 'NL48']
else:
    region_sum = ['CONUS']

# list of summary tables for the chemical
all_table = [v for v in os.listdir(final_path) if v.endswith('.csv') and not v.endswith('_SummaryIntervals.csv')]
dates = [v.split("_")[3].replace('.csv','') for v in all_table ]
if max(dates) != date:
    date = max(dates)
list_table = []

for table in all_table:
    chk_date = table.split("_")[3].replace('.csv','')
    if str(chk_date) == str(date):
        list_table.append(table)

print ("The tables to be summarize include{0}".format(list_table))

if os.path.exists(os.path.dirname(final_path) + os.sep + "SummaryTables_1"):
    list_folder = os.listdir(os.path.dirname(final_path))
    summary = [v for v in list_folder if v.startswith('SummaryTables_')]
    count = []
    for v in summary:
        try:
            count.append(int(v.split("_")[1]))
        except ValueError:
            pass

        max_folder = max(count) +1
    out_folder= "SummaryTables_"+str(max_folder)
else:
    out_folder = "SummaryTables_1"

os.mkdir(os.path.dirname(final_path) +os.sep+out_folder)

out_location = os.path.dirname(final_path) +os.sep+out_folder  # output location




for tbl in list_table:
    # loops over summary tables for the chemical
    print('\nCreating summary interval table for all uses...{0}'.format(tbl))
    collapsed_df = base_sp_df  # empty  dataframe to collapse the drifter interval based on the bins
    try:
        sp_table_df = pd.read_csv(final_path + os.sep + tbl , dtype=object,encoding ='ISO-8859-1')  # current summary table
    except IOError:
        sp_table_df = pd.read_csv(out_location + os.sep + tbl, dtype=object,encoding ='ISO-8859-1')  # current summary table
    if len(sp_table_df) == 0:
        continue
    sp_table_df.fillna(0, inplace=True)  # fills blanks
    # drops any 'Unnamed' columns theses are index cols add in when processing the output
    [sp_table_df.drop(m, axis=1, inplace=True) for m in sp_table_df.columns.values.tolist() if m.startswith('Unnamed')]
    overlap_cols = [v for v in sp_table_df.columns.values.tolist() if v.startswith('CONUS') or v.startswith('NL48')]
    sp_table_df[overlap_cols] = sp_table_df[overlap_cols].apply(pd.to_numeric, errors='coerce')
    sp_table_df['SUM'] = sp_table_df[overlap_cols].sum(axis=1)
    sp_table_df = sp_table_df.loc[sp_table_df['SUM'] > 0]
    sp_table_df.drop('SUM', axis=1, inplace=True)
    # set ID cols as strings
    if 'GEOID' in sp_table_df.columns.values.tolist():
        sp_table_df['GEOID'] = sp_table_df['GEOID'].map(lambda x: str(x) if len(str(x)) == 5 else '0' + str(x)).astype(
            str)
    sp_table_df['EntityID'] = sp_table_df['EntityID'].map(lambda x: str(x).split('.')[0]).astype(str)

    for region in region_sum:
        use_nm=''
        st_or_cnty = []  # list of values found in the summary table need for the group-bys
        for p in sp_table_df.columns.values.tolist():
            if p in ['GEOID', 'STUSPS']:
                st_or_cnty.append(p)
        # loops over the regions - typically for GMO it is only CONUS
        # gets a list of uses found in the table

        uses = []  # empty list of uses for collapsing on the summary intervals set by the bins
        for x in overlap_cols:
            split_value = x.split("_")
            region_abb = split_value[0]
            interval_value = split_value[len(split_value) - 1]
            use_nm = split_value[1]
            if region_abb  + "_"+ use_nm  not in uses:
                uses.append(region_abb  + "_"+ use_nm )
    # adds in PCTs
    for i in sp_table_df.columns.values.tolist():
        if "Upper" in i.split("_") or "Uniform" in i.split("_"):
            name = i.split("_")[0]+"_"+i.split("_")[1]+"_"+i.split("_")[2]+"_"+i.split("_")[3]+"_"+i.split("_")[4]
            if name not in uses:
                uses.append(name)

    # loops over each use in table
    
    for i in uses:
        binned_result = pd.DataFrame()
        print ("Working on {0} in region {1}, and table {2}".format(i, i.split("_")[0], tbl))
        # NOTES SEE ASSUMPTION ABOUT NOT HAVING ANY "_" in the use name part of final use col headers;
        # ie [region]_[use name]_[interval_value] list of current group of column will not populate correctly
        all_cols = [c for c in sp_table_df.columns.values.tolist() if c.startswith(i)]
        use_cols = []
        # removes pct cols for non_pct uses both col headers start with the same prefix
        for p in all_cols:
            if len(p.split("_")) >3 and len(i.split("_"))==2:
                    pass
            else:
                use_cols.append(p)
        grouped_use = sp_table_df[['EntityID'] + st_or_cnty + use_cols].copy()
        previous_col = []
        # loops over each summary value in the bin list and collapase result for all results found in each summary
        # value. Full impact means each interval in bin list in inclusive of the previous, if this false, each
        # result is independent
        for value in bins:
            binned_col = []
            if full_impact:  # direct use is included is drift calculations
                for col in use_cols:
                    get_interval = col.split('_')
                    interval = int(get_interval[(len(get_interval) - 1)])
                    if interval <= value and col not in previous_col:
                        binned_col.append(col)
            elif not full_impact:  # direct use is not included is drift calculations
                if value == bins[0]:
                    for col in use_cols:
                        get_interval = col.split('_')
                        interval = int(get_interval[(len(get_interval) - 1)])
                        if interval == bins[0]:
                            binned_col.append(col)
                else:
                    for col in use_cols:
                        get_interval = col.split('_')
                        interval = int(get_interval[(len(get_interval) - 1)])
                        if interval == 0:
                            continue
                        else:
                            if interval <= value and col not in previous_col:
                                binned_col.append(col)
                for p in binned_col:
                    if p in previous_col:
                        binned_col.remove(p)
                    previous_col.append(p)
            #print (binned_col, value)  # uncomment if you want to see which columns were included for the bin value

            # combines result into the working output df then saves files
            bin_group = grouped_use[['EntityID'] + st_or_cnty + binned_col].copy()
            bin_group.loc[:, binned_col] = bin_group.loc[:, binned_col].apply(pd.to_numeric, errors='coerce')
            if len(binned_result) == 0:
                col_header = ( i) + '_' + str(value)
                bin_group[col_header] = bin_group[binned_col].sum(axis=1)
                binned_result = bin_group[['EntityID'] + st_or_cnty + [col_header]].copy()

            else:
                col_header = ( i) + '_' + str(value)
                bin_group[col_header] = bin_group[binned_col].sum(axis=1)

                use_results_df = bin_group[['EntityID'] + st_or_cnty + [col_header]].copy()
                common_col = [v for v in binned_result.columns.values.tolist() if
                              v in use_results_df.columns.values.tolist()]
                binned_result = pd.merge(binned_result, use_results_df, how='outer', on=common_col)

        common_col = [v for v in binned_result.columns.values.tolist() if v in collapsed_df.columns.values.tolist()]
        collapsed_df = pd.merge(collapsed_df, binned_result, how='outer', on=common_col)
        # saves final tables
    out_csv = out_location + os.sep + tbl.replace('.csv', '_SummaryIntervals.csv')  # update files name
    collapsed_df.drop_duplicates(inplace=True)  # drop any duplicates
    collapsed_df.fillna(0, inplace=True)  # fill blanks with 0s
    use_cols = [v for v in collapsed_df.columns.values.tolist()  if v.startswith('CONUS') or v.startswith('NL48')]
    drop_list = [ m for m in use_cols if m.split("_")[1] =="Federal Lands"]

    if len(drop_list) > 0:
        if 'NL48_Federal Lands_0' in drop_list:
            drop_list.remove('NL48_Federal Lands_0')
        if 'CONUS_Federal Lands_0' in drop_list:
            drop_list.remove('CONUS_Federal Lands_0')
        collapsed_df.drop(drop_list, axis=1, inplace=True)
        # print (drop_list)
    if os.path.exists(out_csv):
            os.remove(out_csv)
    if include_coa:
        #appends pre-processed coa usage info
        
        if "Cnty"  in out_csv.split("_"):
            pass
        elif "State" in out_csv.split("_"):
            pass
        else:
            
            collapsed_df = pd.merge(collapsed_df,coa_usage, on='EntityID', how='left')
    collapsed_df.to_csv(out_csv, encoding ='ISO-8859-1')  # save output
    print ("Saved table: {0}".format(out_csv))

# Elapse clock
end = datetime.datetime.now()
print ("End Time: " + end.ctime())
elapsed = end - start_time
print ("Elapsed  Time: " + str(elapsed))
