import os
import sys
import pandas as pd
# Set the chemical name and the location of your input, output, directory here.
chemical_name = 'AllUDLs'
input_dir = r"C:\Users\Public\ESA_SpeciesUpdate_2023\UDLOverlapTool\input"
output_dir = r"C:\Users\Public\ESA_SpeciesUpdate_2023\UDLOverlapTool\output_yourchemical"

# overwrite boolean - True you existing results will be over-written, set to false tables were already generated for
# some uses, and you want to add new ones, a use layer was updated, etc.
# NOTE: When TRUE all tables will be recalculated.
overwrite_inter_data = True

# ### Range or Critical Run
# True you are completing a run for the range of species False critical habitat for a species
# script will need to be parameterized and executes individually for the range and critical habitat
# True = Range Files False = Critical Habitat
range_run = True


# ##### Taxa of Concern
# Remove taxa that are not of concern for your chemical
# NOTE when you remove the taxa these species will come up as 0% overlap in the tables because there is no risk concern
# if you want to see the overlap even if there is no risk include taxa in the list

# all taxa: 'Amphibians','Arachnids','Birds','Clams', 'Conifers', 'Coral','Crustaceans','Ferns','Fishes',
# 'Flowering Plants','Insects', 'Lichens','Mammals', 'Reptiles', 'Snails'

# Aquatic Inverts include: 'Clams', 'Coral' 'Crustaceans', 'Insects', 'Snails'
# Terrestrial Inverts include:'Arachnids', 'Insects', 'Snails'
# Plants include: 'Conifers', 'Ferns','Flowering Plants', 'Lichens'
# Other terrestrial taxa: 'Birds', 'Mammals', 'Reptiles'
# Other aquatic taxa: 'Fishes'
# Other mixed phase taxa: 'Amphibians'

taxa_concern = ['Amphibians', 'Arachnids', 'Birds', 'Clams', 'Conifers', 'Corals', 'Crustaceans', 'Ferns', 'Fishes',
                'Flowering Plants', 'Insects', 'Lichens', 'Mammals', 'Reptiles', 'Snails']

# ### Regional Restrictions
# List of regions to be included
# full list - ['CONUS','AK', 'AS', 'CNMI', 'GU','HI',  'PR', 'VI']
# CONUS - Conterminous United States
# AK - Alaska
# AS - American Samoa
# CNMI - Commonwealth of the Marinas
# GU - Guam
# HI - Hawaii
# PR - Puerto Rico

regions = ['CONUS', 'AK', 'AS','CNMI', 'GU', 'HI',  'PR']#'CONUS', 'AK', 'AS', 'CNMI', 'GU', 'HI',  'PR',
# ### State Restrictions
# if you want to limit the overlap to specific states based on the registration you can do that here using the
# STUSPS two letter abbreviation for the state
# Format: place the state abbreviate between the bracket in quotation marks, separated by commas
# eg: list_pol_ids = ['AR','FL','MS']
exclusion_state = True # change to false if you are providing a list of state to include
list_pol_ids = []#'AL','FL','GA','NC','SC','TX'
exclusion_cnty = True # change to false if you are providing a list of counties to include
list_geoid_ids = []


# ### Outputs that will be generated:
# Outputs can include tables for species, species by state and species by state and county
# add state ('STUSPS') and/or cnty ('GEOID')to the list below if you wish to generate those tables
# separated by commas and in quotation marks eg, ['EntityID', 'STUSPS', 'GEOID'] or ['EntityID', 'STUSPS']
# NOTE the cnty tables take the most time to generate, only generate if you need them
grouping_col = ['EntityID','STUSPS']#'EntityID','STUSPS', 'GEOID'

# ### Drift and runoff considerations
# list of distances from the field to include in output
# NOTE all distance are in METERS and not feet
# Recommendation - to start use the bins = [0, 30, 305, 792, 1600]
# 0 - Represents on field
# 30 - directly adjacent / runoff to terrestrial habitats
# 305 - limits of spray drift for ground application 305 meters = 1,000 feet
# 792 - limits of spray drift for aerial applications 792 meters = 2,600 feet
# 1600 - runoff to wetland habitat
# If you know you spray distance limit you can use that rather than the 305 or 792. All distances between 0 and 1500
# are available in a 30-meter step, i.e. 0, 30, 60, 90
# remove drift or runoff values if they are not a concern for your chemical; ie you only have ground application
bins =  [0,30,60,90,120,150,305, 792,1600]
# if drift values should include use + drift True if direct use and drift should be separate false. Default for this
# parameter is True
full_impact = True

# ### On/Off Field considerations:
# Set parameter to True if you have a registered use that falls into each category and False if you don't
cult_crop = True   # cultivated crops
orchards_crops = True  # orchards use
pastures = True  # pasture, alfalfa, range land use
residential = True # residential or developed use
forest = True  # Forest or Forestry uses
row = True   # Right of way uses


# ### Chemical use look-up -used to set standard column headers
# This is specific to your chemical, you place your chemical's look-up table in the input_direct and replace the
# 'Uses_lookup_June2021.csv' to your file name
look_up_use = input_dir + os.sep + 'Test_Uses_2024.csv'
pctn_table = input_dir + os.sep + 'Example_PCTs_ForOverlap.csv'

# preprocessed CoA usage for chemical class
# If you would like to append the CoA usage information to your overlap table include the Excel file name for your chemical class
# If you do not want to append the CoA usage information leave this blank
###### Herbicides_SpeciesSummaryTables.xlsx
###### Fungicides_SpeciesSummaryTables.xlsx
###### Insecticides_SpeciesSummaryTables.xlsx
###### Nematicides_SpeciesSummaryTables.xlsx

coa_usage_excel = input_dir + os.sep + "Herbicides_SpeciesSummaryTables.xlsx"

# ### Location of the chemical independent overlap and standard species input tables
# these paths represent the standard chemical independent overlap. If you used the standard inputs and keep the
# file directory structure standard you should not need to make any updates to these paths, but is a good idea to
# verify them
# NOTE : Range or CriticalHabitat needs to be included in the path to the files

if range_run:  # inputs values for range files
    in_results = input_dir + os.sep + r'RawResults\Range'
    # directory for species look-up tables - key_col is the header used as the zone in the species runs and will be
    # joined to the VALUE field from the raw overlap table
    in_look_up = input_dir + os.sep + r"Lookup_R_Clipped_Union_CntyInter_HUC2ABInter_20230112"
    # typically, HUCID unless you are doing a habitat refinement
    key_col = 'HUCID'  # other options 'ZoneID','InterID', 'HUCID', 'VALUE'
    # Habitat refinement using GAP - updated to True when using this overlap. This is typically False
    include_habitat = False
    # species range acres table
    in_acres_table = input_dir + os.sep + 'R_Acres_Pixels_20240402.csv'

else:  # inputs values for critical habitat files
    in_results = input_dir + os.sep + r'RawResults\CriticalHabitat'
    # directory for species look-up tables - key_col is the header used as the zone in the species runs and will be
    # joined to the VALUE field from the raw overlap table
    in_look_up = input_dir + os.sep + r"Lookup_CH_Clipped_Union_CntyInter_HUC2ABInter_20231201"
    # typically, HUCID unless you are doing a habitat refinement
    key_col = 'HUCID'  # other options 'ZoneID','InterID', 'HUCID', 'VALUE'
    # Never run habitat refinement on critical habitat; this will always be False
    include_habitat = False
    # species critical habitat acres table
    in_acres_table = input_dir + os.sep + 'CH_Acres_Pixels_20240401.csv'
    # on/off adjustments are not applied to the critical habitat all of these parameters are re-set to False
    cult_crop = False
    orchards_crops = False
    pastures = False
    residential = False
    forest = False
    row = False

# Master species list: Be sure to keep a record of the master species used for the assessment because includes
# the specific spatial files used for the specific results and these are routinely updated over time.
master_list = input_dir + os.sep + 'MasterSpeciesList_12012024.csv'

# columns from the species list to be included in the output table
col_include_output = ['EntityID', 'Common Name', 'Scientific Name', 'ESA Listing Status', 'EPA Species Group','Species Group','Phase' ,'Lead Agency', "Critical Habitat Type","CH_GIS",'Range_GIS',
'Qualitative Species','Aquatic Species']

on_off_excel = os.path.join(input_dir, "ON_OFF_calls_Jan2024.xlsx")
sheet_name_on_off = 'ON_OFF_Calls'


# ##################### Do not edit - these are the input generated based on the user inputs above
# Identify python version
if sys.version_info.major >= 3:
    python_version_three = True
else:
    python_version_three = False

# ###### usage infomation
in_pol_bndry_overlap = input_dir + os.sep + "UDL_Totals_State_Cnty"
st_abb_lookup = input_dir + os.sep + "STATEFP_lookup.csv"

# creates output folder if it doesn't exist
if not os.path.exists(output_dir):
    os.mkdir(output_dir)
#  True only include result up to the highest bin value in the overlap table. Runs much faster/False all interval up to
# 1500 are included then summarize at step 5
only_include_bins = True

# folder name for your tabulated output results
name_tabulated_folder = 'Tabulated_species'

#  Output location species summary
# file suffix used for the results files
if range_run:
    tabulated = output_dir + os.sep + 'Range' + os.sep + name_tabulated_folder
    file_type = 'Range'
else:
    tabulated = output_dir + os.sep + 'CriticalHabitat' + os.sep + name_tabulated_folder
    file_type = 'CriticalHabitat'

# Output location species/cnty summary
out_dir_summaries = os.path.dirname(tabulated)
# output location merge use across all species groups
if len(grouping_col) == 3:
    tabulated_merge = out_dir_summaries + os.sep + 'Overlap_byCountyMerge'
else:
    tabulated_merge = out_dir_summaries + os.sep + 'Overlap_byStateMerge'
# output percent overlap tables for use
per_overlap_path = tabulated_merge.replace('Merge', 'PercentOverlap')
# output path final tables for chemical
final_path = os.path.dirname(per_overlap_path)
id_value = file_type + "_" + chemical_name

if range_run:
    all_range_taxa = ['Amphibians', 'Arachnids', 'Birds', 'Clams', 'Conifers', 'Corals', 'Crustaceans', 'Ferns',
                      'Fishes', 'Flowering Plants', 'Insects', 'Lichens', 'Mammals', 'Reptiles', 'Snails']
    drop_taxa = [v for v in all_range_taxa if v not in taxa_concern]
    species_dict = {'Amphibians': 'r_amphib',
                    'Arachnids': 'r_arachn',
                    'Birds': 'r_birds',
                    'Clams': 'r_clams',
                    'Conifers': 'r_conife',
                    'Corals': 'r_corals',
                    'Crustaceans': 'r_crusta',
                    'Ferns': 'r_ferns',
                    'Fishes': 'r_fishes',
                    'Flowering Plants': 'r_flower',
                    'Insects': 'r_insect',
                    'Lichens': 'r_lichen',
                    'Mammals': 'r_mammal',
                    'Reptiles': 'r_reptil',
                    'Snails': 'r_snails'}
else:
    all_ch_taxa = ['Amphibians', 'Arachnids', 'Birds', 'Clams', 'Corals','Crustaceans', 'Ferns', 'Fishes', 'Flowering Plants',
                   'Insects', 'Mammals', 'Reptiles', 'Snails']
    drop_taxa = [v for v in all_ch_taxa if v not in taxa_concern]
    species_dict = {'Amphibians': 'ch_amphi',
                    'Arachnids': 'ch_arach',
                    'Birds': 'ch_birds',
                    'Clams': 'ch_clams',
                    'Conifers': 'ch_conife',
                    'Corals': 'ch_coral',
                    'Crustaceans': 'ch_crust',
                    'Ferns': 'ch_ferns',
                    'Fishes': 'ch_fishe',
                    'Flowering Plants': 'ch_flowe',
                    'Insects': 'ch_insec',
                    'Mammals': 'ch_mamma',
                    'Reptiles': 'ch_repti',
                    'Snails': 'ch_snail'}
for taxa in drop_taxa:
    del species_dict[taxa]
species_prefix = species_dict.values()


